'use strict';

var leserNebulaDsl = require('@sj1/leser-nebula-dsl');
var language = require('@codemirror/language');

/**
A language provider based on the [Lezer Java
parser](https://github.com/lezer-parser/ndsl), extended with
highlighting and indentation information.
*/
const ndslLanguage = language.LRLanguage.define({
    name: "ndsl",
    parser: leserNebulaDsl.parser.configure({
        props: [
            language.indentNodeProp.add({
                IfStatement: language.continuedIndent({ except: /^\s*({|else\b)/ }),
                TryStatement: language.continuedIndent({ except: /^\s*({|catch|finally)\b/ }),
                LabeledStatement: language.flatIndent,
                SwitchBlock: context => {
                    let after = context.textAfter, closed = /^\s*\}/.test(after), isCase = /^\s*(case|default)\b/.test(after);
                    return context.baseIndent + (closed ? 0 : isCase ? 1 : 2) * context.unit;
                },
                Block: language.delimitedIndent({ closing: "}" }),
                BlockComment: () => null,
                Statement: language.continuedIndent({ except: /^{/ })
            }),
            language.foldNodeProp.add({
                ["Block SwitchBlock ClassBody ElementValueArrayInitializer ModuleBody EnumBody " +
                    "ConstructorBody InterfaceBody ArrayInitializer"]: language.foldInside,
                BlockComment(tree) { return { from: tree.from + 2, to: tree.to - 2 }; }
            })
        ]
    }),
    languageData: {
        commentTokens: { line: "//", block: { open: "/*", close: "*/" } },
        indentOnInput: /^\s*(?:case |default:|\{|\})$/
    }
});
/**
Java language support.
*/
function ndsl() {
    return new language.LanguageSupport(ndslLanguage);
}

exports.ndsl = ndsl;
exports.ndslLanguage = ndslLanguage;
